#!/usr/bin/python3

import os.path
import shutil
import unidecode

import xdg.DesktopEntry
import xdg.Menu

import gi
gi.require_version("Gtk", "3.0")
gi.require_version("MateDesktop", "2.0")
from gi.repository import Gtk, Gdk, GdkPixbuf, GLib, GObject, Pango, MateDesktop

from plugins.execute import Execute
from plugins.filemonitor import monitor as filemonitor


class IconManager(GObject.GObject):

    __gsignals__ = {
        "changed" : (GObject.SignalFlags.RUN_LAST, None, ())
   }

    def __init__(self):
        GObject.GObject.__init__(self)
        self.icons = {}
        self.count = 0

        # Some apps don't put a default icon in the default theme folder, so we will search all themes
        # def createTheme(d):
        #     theme = Gtk.IconTheme()
        #     theme.set_custom_theme(d)
        #     return theme

        # This takes to much time and there are only a very few applications that use icons from different themes
        #self.themes = map( createTheme, [d for d in os.listdir("/usr/share/icons") if os.path.isdir(os.path.join("/usr/share/icons", d))])

        self.defaultTheme = Gtk.IconTheme.get_default()
        #defaultKdeTheme = createTheme("kde.default")

        # Setup and clean up the temp icon dir
        configDir = GLib.get_user_config_dir()
        self.iconDir = os.path.join(configDir, "mintmenu")
        if not os.path.exists(self.iconDir):
            os.makedirs(self.iconDir)
        contents = os.listdir(self.iconDir)
        for fn in contents:
            os.remove(os.path.join(self.iconDir, fn))

        self.defaultTheme.append_search_path(self.iconDir)
        # Themes with the same content as the default them aren't needed
        #self.themes = [theme for theme in self.themes if  theme.list_icons() != defaultTheme.list_icons()]

        #self.themes = [self.defaultTheme, defaultKdeTheme]
        self.themes = [self.defaultTheme]

        # Listen for changes in the themes
        # for theme in self.themes:
        #     theme.connect("changed", self.themeChanged)

    def getIcon(self, iconName, iconSize):
        if not iconName:
            return None

        try:
            iconFileName = ""
            realIconName = ""
            needTempFile = False
            #[iconWidth, iconHeight] = self.getIconSize(iconSize)
            if iconSize <= 0:
                return None

            elif os.path.isabs(iconName):
                iconFileName = iconName
                needTempFile = True
            else:
                if iconName[-4:] in [".png", ".xpm", ".svg", ".gif"]:
                    realIconName = iconName[:-4]
                else:
                    realIconName = iconName

            if iconFileName and needTempFile and os.path.exists(iconFileName):
                tmpIconName = iconFileName.replace("/", "-")
                realIconName = tmpIconName[:-4]
                if not os.path.exists(os.path.join(self.iconDir, tmpIconName)):
                    shutil.copyfile(iconFileName, os.path.join(self.iconDir, tmpIconName))
                    self.defaultTheme.append_search_path(self.iconDir)

            image = Gtk.Image.new_from_icon_name(realIconName, Gtk.IconSize.DND)
            image.set_pixel_size(iconSize)
            return image

        except Exception as e:
            print("Exception %s: %s" % (e.__class__.__name__, e))
            return None

    def themeChanged(self, theme):
        self.emit("changed")

GObject.type_register(IconManager)

class easyButton(Gtk.Button):

    def __init__(self, iconName, iconSize, labels=None, buttonWidth=-1, buttonHeight=-1, ellipsis=True):
        GObject.GObject.__init__(self)
        self.connections = []
        self.iconName = iconName
        self.iconSize = iconSize
        self.showIcon = True
        self.ellipsis = ellipsis

        self.set_relief(Gtk.ReliefStyle.NONE)
        self.set_size_request(buttonWidth, buttonHeight)

        Align1 = Gtk.Alignment.new(0, 0.5, 1.0, 0)
        HBox1 = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.labelBox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=2)

        self.buttonImage = self.getIcon(self.iconSize)
        if not self.buttonImage:
            self.buttonImage = Gtk.Image()
            self.buttonImage.set_size_request(self.iconSize, self.iconSize)

        self.image_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.image_box.pack_start(self.buttonImage, False, False, 5)
        self.image_box.show_all()
        HBox1.pack_start(self.image_box, False, False, 0)

        if labels:
            for label in labels:
                if isinstance(label, str):
                    self.addLabel(label)
                elif isinstance(label, list):
                    self.addLabel(label[0], label[1])

        self.labelBox.show()
        HBox1.pack_start(self.labelBox , True, True, 0)
        HBox1.show()
        Align1.add(HBox1)
        Align1.show()
        self.add(Align1)

        self.connectSelf("destroy", self.onDestroy)
        self.connect("released", self.onRelease)
        # Reload icons when the theme changed
        # self.themeChangedHandlerId = iconManager.connect("changed", self.themeChanged)

    def connectSelf(self, event, callback):
        self.connections.append(self.connect(event, callback))

    def onRelease(self, widget):
        widget.get_style_context().set_state(Gtk.StateFlags.NORMAL)

    def onDestroy(self, widget):
        self.buttonImage.clear()
        # iconManager.disconnect(self.themeChangedHandlerId)
        for connection in self.connections:
            self.disconnect(connection)
        del self.connections

    def addLabel(self, text, styles = None):
        label = Gtk.Label()
        if "<b>" in text or "<span" in text:
            label.set_markup(text.replace('&', '&amp;')) # don't remove our pango
        else:
            label.set_text(text)

        if styles:
            labelStyle = Pango.AttrList()
            for attr in styles:
                labelStyle.insert(attr)
            label.set_attributes(labelStyle)

        if self.ellipsis:
            label.set_ellipsize(Pango.EllipsizeMode.END)
        label.set_alignment(0.0, 1.0)
        label.set_max_width_chars(0)
        label.show()
        self.labelBox.pack_start(label , True, True, 0)

    def getIcon(self, iconSize):
        if not self.iconName:
            return None
        icon = iconManager.getIcon(self.iconName, iconSize)
        return icon

    def setIcon(self, iconName):
        self.iconName = iconName
        self.iconChanged()

    # IconTheme changed, setup new button icons
    def themeChanged(self, theme):
        self.iconChanged()

    def iconChanged(self):
        icon = self.getIcon(self.iconSize)
        self.buttonImage.destroy()
        if icon:
            self.buttonImage = icon
            self.image_box.pack_start(self.buttonImage, False, False, 5)
            self.image_box.show_all()
        else:
            #[iW, iH] = iconManager.getIconSize(self.iconSize)
            self.buttonImage.set_size_request(self.iconSize, self.iconSize )

    def setIconSize(self, size):
        self.iconSize = size
        icon = self.getIcon(self.iconSize)
        self.buttonImage.destroy()
        if icon:
            self.buttonImage = icon
            self.image_box.pack_start(self.buttonImage, False, False, 5)
            self.image_box.show_all()
        elif self.iconSize:
            #[iW, iH] = iconManager.getIconSize(self.iconSize)
            self.buttonImage.set_size_request(self.iconSize, self.iconSize )

class ApplicationLauncher(easyButton):

    def __init__(self, desktopFile, iconSize):

        if isinstance(desktopFile, xdg.Menu.MenuEntry):
            desktopItem = desktopFile.DesktopEntry
            desktopFile = desktopItem.filename
            self.appDirs = desktopFile.AppDirs
        elif isinstance(desktopFile, xdg.Menu.DesktopEntry):
            desktopItem = desktopFile
            desktopFile = desktopItem.filename
            self.appDirs = [os.path.dirname(desktopItem.filename)]
        else:
            desktopItem = xdg.DesktopEntry.DesktopEntry(desktopFile)
            self.appDirs = [os.path.dirname(desktopFile)]

        self.desktopFile = desktopFile
        self.startupMonitorId = 0

        self.loadDesktopEntry(desktopItem)

        self.desktopEntryMonitors = []

        base = os.path.basename(self.desktopFile)
        for directory in self.appDirs:
            self.desktopEntryMonitors.append(filemonitor.addMonitor(os.path.join(directory, base) , self.desktopEntryFileChangedCallback))

        easyButton.__init__(self, self.appIconName, iconSize)
        self.setupLabels()

        # Drag and Drop
        self.connectSelf("drag-data-get", self.dragDataGet)

        targets = (Gtk.TargetEntry.new("text/plain", 0, 100), Gtk.TargetEntry.new("text/uri-list", 0, 101))
        self.drag_source_set(Gdk.ModifierType.BUTTON1_MASK, targets, Gdk.DragAction.COPY)
        self.drag_source_set_icon_name(self.iconName)

        self.connectSelf("focus-in-event", self.onFocusIn)
        self.connectSelf("focus-out-event", self.onFocusOut)
        self.connectSelf("clicked", self.execute)

    def loadDesktopEntry(self, desktopItem):
        try:
            self.appName = desktopItem.getName()
            self.appGenericName = desktopItem.getGenericName()
            self.appComment = desktopItem.getComment()
            self.appExec = desktopItem.getExec().replace('\\\\', '\\')
            self.appIconName = desktopItem.getIcon()
            self.appCategories = desktopItem.getCategories()
            self.appMateDocPath = desktopItem.get("X-MATE-DocPath") or ""
            self.useTerminal = desktopItem.getTerminal()
            self.appPath = desktopItem.getPath()
            self.prefersOffload = desktopItem.get("PrefersNonDefaultGPU", "Desktop Entry", type="boolean")

            if not self.appMateDocPath:
                self.appKdeDocPath      = desktopItem.getDocPath() or ""

            self.appName            = self.appName.strip()
            self.appGenericName     = self.appGenericName.strip()
            self.appComment         = self.appComment.strip()

            basename = os.path.basename(self.desktopFile)
            self.startupFilePath = os.path.join(os.path.expanduser("~"), ".config/autostart", basename)
            if self.startupMonitorId:
                filemonitor.removeMonitor(self.startupMonitorId )
            if os.path.exists(self.startupFilePath):
                self.startupMonitorId = filemonitor.addMonitor(self.startupFilePath, self.startupFileChanged)

        except Exception as e:
            print(e)
            self.appName            = ""
            self.appGenericName     = ""
            self.appComment         = ""
            self.appExec            = ""
            self.appIconName        = ""
            self.appCategories      = ""
            self.appDocPath         = ""
            self.startupMonitorId   = 0

    def onFocusIn(self, widget, event):
        self.set_relief(Gtk.ReliefStyle.HALF)

    def onFocusOut(self, widget, event):
        self.set_relief(Gtk.ReliefStyle.NONE)

    def setupLabels(self):
        self.addLabel(self.appName)

    def filterText(self, text):
        keywords = self.strip_case_and_accents(text).split()
        appName = self.strip_case_and_accents(self.appName)
        appGenericName = self.strip_case_and_accents(self.appGenericName)
        appComment = self.strip_case_and_accents(self.appComment)
        appExec = self.strip_case_and_accents(self.appExec)
        for keyword in keywords:
            if keyword != "" and appName.find(keyword) == -1 and appGenericName.find(keyword) == -1 and appComment.find(keyword) == -1 and appExec.find(keyword) == -1:
                self.hide()
                return False
        self.show()
        return True

    def strip_case_and_accents(self, string):
        if isinstance(string, str):
            try:
                value = unidecode.unidecode(string.lower())
            except:
                pass
        return value

    def getTooltip(self):
        tooltip = self.appName
        if self.appComment != "" and self.appComment != self.appName:
            tooltip = tooltip + "\n" + self.appComment

        return tooltip

    def dragDataGet(self, widget, context, selection, targetType, eventTime):
        if targetType == 100: # text/plain
            selection.set_text("'" + self.desktopFile + "'", -1)
        elif targetType == 101: # text/uri-list
            if self.desktopFile[0:7] == "file://":
                selection.set_uris([self.desktopFile])
            else:
                selection.set_uris(["file://" + self.desktopFile])

    def execute(self, *args, **kwargs):
        if self.appExec:
            if self.useTerminal:
                cmd = "x-terminal-emulator -e \"" + self.appExec + "\""
                if os.path.exists("/usr/bin/mate-terminal"):
                    cmd = "mate-terminal -e \"" + self.appExec + "\""
                Execute(cmd, self.appPath)
            else:
                offload = False

                try:
                    offload = kwargs["offload"]
                except KeyError:
                    pass

                if self.prefersOffload:
                    offload = True

                Execute(None, desktopFile=self.desktopFile, offload=offload)

    def uninstall(self, *args):
        Execute("mint-remove-application " + self.desktopFile)

    # IconTheme changed, setup new icons for button and drag 'n drop
    def iconChanged(self):
        easyButton.iconChanged(self)

        icon = self.getIcon(Gtk.IconSize.DND)
        if icon:
            iconName, size = icon.get_icon_name()
            self.drag_source_set_icon_name(iconName)

    def startupFileChanged(self, *args):
        self.inStartup = os.path.exists(self.startupFilePath)

    def addToStartup(self):
        startupDir = os.path.join(os.path.dirname(self.startupFilePath))
        if not os.path.exists(startupDir):
            os.makedirs(startupDir)

        shutil.copyfile(self.desktopFile, self.startupFilePath)

        # Remove %u, etc. from Exec entry, because MATE will not replace them when it starts the app
        item = MateDesktop.DesktopItem.new_from_uri(self.startupFilePath, MateDesktop.DesktopItemLoadFlags.ONLY_IF_EXISTS)
        if item:
            r = re.compile("%[A-Za-z]")
            tmp = r.sub("", item.get_string(MateDesktop.DESKTOP_ITEM_EXEC)).strip()
            item.set_string(MateDesktop.DESKTOP_ITEM_EXEC, tmp)
            item.save(self.startupFilePath, 0)

    def removeFromStartup(self):
        if os.path.exists(self.startupFilePath):
            os.remove(self.startupFilePath)

    def removeFromFavourites(self):
        if os.path.exists(self.favouritesFilePath):
            os.remove(self.favouritesFilePath)

    def isInStartup(self):
        #return self.inStartup
        return os.path.exists(self.startupFilePath)

    def onDestroy(self, widget):
        easyButton.onDestroy(self, widget)
        if self.startupMonitorId:
            filemonitor.removeMonitor(self.startupMonitorId)
        for id in self.desktopEntryMonitors:
            filemonitor.removeMonitor(id)

    def desktopEntryFileChangedCallback(self):
        GLib.timeout_add(200, self.onDesktopEntryFileChanged)

    def onDesktopEntryFileChanged(self):
        exists = False
        base = os.path.basename(self.desktopFile)
        for directory in self.appDirs:
            if os.path.exists(os.path.join(directory, base)):
                # print os.path.join(dir, base), self.desktopFile
                self.loadDesktopEntry(xdg.DesktopEntry.DesktopEntry(os.path.join(directory, base)))
                for child in self.labelBox:
                    child.destroy()

                self.iconName = self.appIconName

                self.setupLabels()
                self.iconChanged()
                exists = True
                break

        if not exists:
            # FIXME: What to do in this case?
            self.destroy()
        return False

class MenuApplicationLauncher(ApplicationLauncher):

    def __init__(self, desktopFile, iconSize, category, showComment, highlight=False):

        self.showComment = showComment
        self.appCategory = category
        self.highlight = highlight

        ApplicationLauncher.__init__(self, desktopFile, iconSize)

    def filterCategory(self, category):
        if self.appCategory == category or category == "":
            self.show()
        else:
            self.hide()

    def setupLabels(self):
        appName = self.appName
        appComment = self.appComment
        if self.highlight:
            try:
                appName = "<b>%s</b>" % appName
                appComment = "<b>%s</b>" % appComment
            except Exception as e:
                print(e)

        if self.showComment and self.appComment != "":
            if self.iconSize <= 2:
                self.addLabel('<span size="small">%s</span>\n<span size="x-small">%s</span>' %
                    (appName, appComment))
            else:
                self.addLabel('%s\n<span size="small">%s</span>' %
                    (appName, appComment))
        else:
            self.addLabel(appName)

    def execute(self, *args, **kwargs):
        self.highlight = False
        for child in self.labelBox:
            child.destroy()
        self.setupLabels()
        return super(MenuApplicationLauncher, self).execute(*args, **kwargs)

    def setShowComment(self, showComment):
        self.showComment = showComment
        for child in self.labelBox:
            child.destroy()
        self.setupLabels()

class FavApplicationLauncher(ApplicationLauncher):

    def __init__(self, desktopFile, iconSize, swapGeneric = False):

        self.swapGeneric = swapGeneric

        ApplicationLauncher.__init__(self, desktopFile, iconSize)

    def setupLabels(self):
        if self.appGenericName:
            if self.swapGeneric:
                self.addLabel('<span weight="bold">%s</span>' % self.appName)
                self.addLabel(self.appGenericName)
            else:
                self.addLabel('<span weight="bold">%s</span>' % self.appGenericName)
                self.addLabel(self.appName)
        else:
            self.addLabel('<span weight="bold">%s</span>' % self.appName)
            if self.appComment != "":
                self.addLabel(self.appComment)
            else:
                self.addLabel("")

    def setSwapGeneric(self, swapGeneric):
        self.swapGeneric = swapGeneric
        for child in self.labelBox:
            child.destroy()

        self.setupLabels()

class CategoryButton(easyButton):

    def __init__(self, iconName, iconSize, labels , f):
        easyButton.__init__(self, iconName, iconSize, labels, ellipsis=False)
        self.filter = f

iconManager = IconManager()



#!/usr/bin/python3

import os

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

from plugins.easybuttons import ApplicationLauncher

home = os.path.expanduser("~")
recentApps = []
settings = None # set by recent plugin

mintMenuWin = None
recentAppBox = None
numentries = 10
iconSize = 16

def recentAppsAdd(recentAppsButton):
    if recentAppsButton:
        recentApps.insert(0, recentAppsButton)
        counter = 0
        for recentApp in recentApps:
            if counter != 0 and (recentApp.desktopFile == recentAppsButton.desktopFile or counter >= numentries):
                del recentApps[counter]
            counter = counter + 1

def recentAppsSave():
    new_recent_apps = []

    for recentApp in recentApps:
        if not hasattr(recentApp, "type") or recentApp.type == "location":
            new_recent_apps.append("location:" + recentApp.desktopFile)
        else:
            new_recent_apps.append(recentApp.type)

    settings.set_strv("recent-apps-list", new_recent_apps)

def recentAppBuildLauncher(location):
    try:
        # For Folders and Network Shares
        location = "".join(location.split("%20"))

        # ButtonIcon = None

        # if location.startswith("file"):
        #     ButtonIcon = "mate-fs-directory"

        # if location.startswith("smb") or location.startswith("ssh") or location.startswith("network"):
        #     ButtonIcon = "mate-fs-network"

        #For Special locations
        if location == "x-nautilus-desktop:///computer":
            location = "/usr/share/applications/nautilus-computer.desktop"
        elif location == "x-nautilus-desktop:///home":
            location =  "/usr/share/applications/nautilus-home.desktop"
        elif location == "x-nautilus-desktop:///network":
            location = "/usr/share/applications/network-scheme.desktop"
        elif location.startswith("x-nautilus-desktop:///"):
            location = "/usr/share/applications/nautilus-computer.desktop"

        if location.startswith("file://"):
            location = location[7:]
        if os.path.exists(location):
            appButton = ApplicationLauncher(location, iconSize)
            if appButton.appExec:
                appButton.show()
                appButton.connect("clicked", applicationButtonClicked)
                appButton.type = "location"
                return appButton
        print("RecentApp: %s not found." % location)
    except Exception as e:
        print("File in recentapp not found: '%s': %s" % (location, e))

    return None

def buildRecentApps():
    del recentApps[:]
    try:
        recent_apps = settings.get_strv("recent-apps-list")

        for app in recent_apps:
            app = app.strip()

            if app[0:9] == "location:":
                appButton = recentAppBuildLauncher(app[9:])
            else:
                if (app.endswith(".desktop")):
                    appButton = recentAppBuildLauncher(app)
                else:
                    appButton = None

            if appButton:
                recentApps.append(appButton)
    except Exception as e:
        print(e)
    return recentApps

def doRecentApps():
    if recentAppBox is not None:
        # recentAppBox is initiated by the recent plugin
        # only build UI widgets if it's enabled
        for i in recentAppBox.get_children():
            i.destroy()

        # recent apps
        buildRecentApps()
        for AButton in recentApps:
            AButton.set_size_request(200, -1)
            AButton.set_relief(Gtk.ReliefStyle.NONE)
            recentAppBox.pack_start(AButton, False, True, 0)

    return True

def applicationButtonClicked(widget):
    mintMenuWin.hide()

    if settings == None:
        return

    recentAppsAdd(widget)
    recentAppsSave()
    doRecentApps()



#!/usr/bin/python3

import sys

import apt_pkg

if len(sys.argv) != 2:
    sys.exit(1)
try:
    apt_pkg.init()
    cache = apt_pkg.Cache()
    package_records = apt_pkg.PackageRecords(cache)
    known_packages = set()
    with open(sys.argv[1], "w") as f:
        for pkg in cache.packages:
            if pkg.selected_state or not pkg.version_list or pkg.name in known_packages:
                continue
            name = pkg.name
            package_records.lookup(pkg.version_list.pop(0).translated_description.file_list.pop(0))
            summary = package_records.short_desc
            description = package_records.long_desc.replace(summary + "\n ", "").replace("\n .\n ", "~~~").replace("\n", "")
            f.write("CACHE###%s###%s###%s\n" % (name, summary, description))
            known_packages.add(name)
except Exception as e:
    print("ERROR###ERROR###ERROR###ERROR")
    print(e)
    sys.exit(1)



#!/usr/bin/python3

import gettext
import locale
import os
import subprocess

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio, Pango

import plugins.recentHelper as RecentHelper
from plugins.execute import Execute

home = os.path.expanduser("~")

# i18n
gettext.install("mintmenu", "/usr/share/linuxmint/locale")
locale.bindtextdomain("mintmenu", "/usr/share/linuxmint/locale")
locale.textdomain("mintmenu")

class pluginclass:
    """ This is the main class for the plugin.
        It MUST be named pluginclass
    """

    def __init__(self, mintMenuWin, toggleButton, de):

        self.Win = mintMenuWin
        self.toggleButton = toggleButton
        self.de = de

        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("mintmenu")
        self.builder.add_from_file("/usr/share/linuxmint/mintmenu/recent.ui")

        #Set 'window' property for the plugin (Must be the root widget)
        self.window = self.builder.get_object("window1")

        #Set 'heading' property for plugin
        self.heading = _("Recently used")

        #This should be the first item added to the window in glade
        self.content_holder = self.builder.get_object("eventbox1")

        self.recentBox = self.builder.get_object("RecentBox")
        self.recentAppBox = self.builder.get_object("RecentApps")
        RecentHelper.recentAppBox = self.recentAppBox

        #self.recentApps = []

        self.recentVBox = self.builder.get_object("vbox1")

        #Specify plugin width
        self.width = 250

        #Plugin icon
        self.icon = 'mate-folder.png'

        self.settings = Gio.Settings("com.linuxmint.mintmenu.plugins.recent")
        RecentHelper.settings = self.settings

        self.migrate_recent_apps()
        self.settings.connect('changed', self.RegenPlugin)

        self.appSettings = Gio.Settings("com.linuxmint.mintmenu.plugins.applications")
        self.appSettings.connect("changed::icon-size", self.RegenPlugin)

        self.FileList=[]
        self.RecManagerInstance = Gtk.RecentManager.get_default()
        self.recentManagerId = self.RecManagerInstance.connect("changed", self.DoRecent)

        self.RegenPlugin()
        self.builder.get_object("RecentTabs").set_current_page(0)

        #Connect event handlers
        self.builder.get_object("ClrBtn").connect("clicked", self.clrmenu)

    @staticmethod
    def wake():
        return

    def destroy(self):
        self.recentBox.destroy()
        self.recentVBox.destroy()
        self.builder.get_object("RecentTabs").destroy()
        self.builder.get_object("ClrBtn").destroy()
        self.content_holder.destroy()
        if self.recentManagerId:
            self.RecManagerInstance.disconnect(self.recentManagerId)

    def RegenPlugin(self, *args, **kargs):
        self.GetGSettingsEntries()

    def migrate_recent_apps(self):
        if self.settings.get_strv("recent-apps-list") != []:
            return

        path = os.path.join(home, ".linuxmint/mintMenu/recentApplications.list")
        if os.path.exists(path):
            with open(path) as f:
                self.settings.set_strv("recent-apps-list", f.readlines())

            try:
                os.unlink(path)
            except:
                pass

    def GetGSettingsEntries(self):
        self.recenth = self.settings.get_int("height")
        self.recentw = self.settings.get_int("width")
        self.numentries = self.settings.get_int("num-recent-docs")
        RecentHelper.numentries = self.numentries
        self.recentfontsize = self.settings.get_int("recent-font-size")

        # Hide vertical dotted separator
        self.hideseparator = self.settings.get_boolean("hide-separator")
        # Plugin icon
        self.icon = self.settings.get_string("icon")
        RecentHelper.iconSize = self.appSettings.get_int("icon-size")
        self.RebuildPlugin()

    def RebuildPlugin(self):
        self.content_holder.set_size_request(self.recentw, self.recenth)
        self.DoRecent()

    def DoRecent(self, *args, **kargs):
        for i in self.recentBox.get_children():
            i.destroy()

        self.recentVBox.set_size_request(self.recentw, self.recenth)
        if len(self.recentBox.get_children()) < self.numentries:
            n=len(self.recentBox.get_children())-1
        else:
            n=self.numentries-1
        while n >= 0:
            self.recentBox.remove(self.recentBox.get_children()[n])
            n-=1

        self.FileList, self.IconList = self.GetRecent()
        loc = 0
        for Name in self.FileList:
            if Name != None:
                self.AddRecentBtn(Name, self.IconList[loc])
            loc = loc + 1

        RecentHelper.doRecentApps()

        return True

    def clrmenu(self, *args, **kargs):
        if self.builder.get_object("RecentTabs").get_current_page() == 0: # files
            self.RecManagerInstance.purge_items()
        else: # apps
            self.settings.reset("recent-apps-list")

        self.DoRecent()

    def AddRecentBtn(self, Name, RecentImage):
        DispName=os.path.basename(Name)

        AButton = Gtk.Button("", "ok", True)
        AButton.remove(AButton.get_children()[0])
        AButton.set_size_request(200, -1)
        AButton.set_relief(Gtk.ReliefStyle.NONE)
        AButton.connect("clicked", self.callback, Name)
        AButton.show()

        Box1 = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5)

        ButtonIcon = Gtk.Image()
        ButtonIcon.set_size_request(20, -1)
        ButtonIcon.set_from_pixbuf(RecentImage)
        Box1.add(ButtonIcon)

        Label1 = Gtk.Label(DispName)
        Label1.set_ellipsize(Pango.EllipsizeMode.END)
        Box1.add(Label1)

        AButton.add(Box1)
        AButton.show_all()

        self.recentBox.pack_start(AButton, False, True, 0)

    def callback(self, widget, filename):
        self.Win.hide()

        try:
            subprocess.check_call(["xdg-open", filename])
        except subprocess.CalledProcessError:
            dialog = Gtk.MessageDialog(self.window, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
            Gtk.MessageType.ERROR, Gtk.ButtonsType.OK, _("The file or location could not be opened."))
            dialog.set_title("mintMenu")
            dialog.run()
            dialog.destroy()

    def GetRecent(self, *args, **kargs):
        FileString=[]
        IconString=[]
        RecentInfo=sorted(self.RecManagerInstance.get_items(), key=lambda item: item.get_modified(), reverse=True)
        count=0
        MaxEntries=self.numentries
        if self.numentries == -1:
            MaxEntries=len(RecentInfo)
        for items in RecentInfo:
            FileString.append(items.get_uri_display())
            IconString.append(items.get_icon(Gtk.IconSize.MENU))
            count+=1
            if count >= MaxEntries:
                break
        return FileString, IconString

    def ButtonClicked(self, widget, event, Exec):
        self.press_x = event.x
        self.press_y = event.y
        self.Exec = Exec

    def ButtonReleased(self, w, ev, ev2):
        if ev.button == 1:
            if not hasattr(self, "press_x") or \
                    not w.drag_check_threshold(int(self.press_x),
                                                int(self.press_y),
                                                int(ev.x),
                                                int(ev.y)):
                if self.Win.pinmenu == False:
                    self.Win.wTree.get_widget("window1").hide()
                if "applications" in self.Win.plugins:
                    self.Win.plugins["applications"].wTree.get_widget("entry1").grab_focus()
                Execute(w, self.Exec)

    @staticmethod
    def do_plugin():
        return



#!/usr/bin/python3

import os
import os.path
import threading
import time
from gi.repository import GLib

try:
    import pyinotify
    hasInotify = True
except ImportError:
    hasInotify = False


if hasInotify:

    class FileMonitor(object):
        def __init__(self):
            self.monitorId = 0
            self.wm = pyinotify.WatchManager()
            self.wdds = {}
            self.callbacks = {}
            self.notifier = pyinotify.ThreadedNotifier(self.wm, self.fileChanged)
            self.notifier.setDaemon(True)
            self.notifier.start()


        def addMonitor(self, filename, callback, args = None):
            try:
                mask = pyinotify.IN_DELETE | pyinotify.IN_CREATE | pyinotify.IN_MODIFY
                mId = self.wm.add_watch(filename, mask, rec = True)[filename]
                if mId >= 0:
                    self.callbacks[mId] = (callback, args)
            except:
                mId = 0
            return mId

        def removeMonitor(self, monitorId):
            if monitorId in self.callbacks:
                self.wm.rm_watch(monitorId)
                del self.callbacks[monitorId]

        def fileChanged(self, event):
            if event.wd in self.callbacks:
                #print event.path
                callback = self.callbacks[event.wd]
                if callback[1]:
                    GLib.idle_add(callback[0], callback[1])
                else:
                    GLib.idle_add(callback[0])

else:

    class _MonitoredFile(object):
        def __init__(self, filename, callback, monitorId, args):
            self.filename = filename
            self.callback = callback
            self.monitorId = monitorId
            self.args = args
            self.exists = os.path.exists(self.filename)
            if self.exists:
                self.mtime = os.stat(filename).st_mtime
            else:
                self.mtime = 0

        def hasChanged(self):
            if os.path.exists(self.filename):
                if not self.exists:
                    self.exists = True
                    self.mtime = os.stat(self.filename).st_mtime
                    return True
                else:
                    mtime = os.stat(self.filename).st_mtime
                    if mtime != self.mtime:
                        self.mtime = mtime
                        return True
            else:
                if self.exists:
                    self.exists = False
                    return True

            return False

    class MonitorThread(threading.Thread):
        def __init__(self, monitor):
            threading.Thread.__init__(self)
            self.monitor = monitor

        def run(self):
            while(1):
                self.monitor.checkFiles()
                time.sleep(1)

    class FileMonitor(object):
        def __init__(self):
            self.monitorId = 0
            self.monitoredFiles = []
            self.monitorThread = MonitorThread(self)
            self.monitorThread.setDaemon(True)
            self.monitorThread.start()

        def addMonitor(self, filename, callback, args = None):
            self.monitorId += 1
            self.monitoredFiles.append(_MonitoredFile(filename, callback, self.monitorId, args))
            return self.monitorId

        def removeMonitor(self, monitorId):
            for monitored in self.monitoredFiles:
                if monitorId == monitored.monitorId:
                    self.monitoredFiles.remove(monitored)
                    break

        def checkFiles(self):
            for monitored in self.monitoredFiles:
                if monitored.hasChanged():
                    if monitored.args:
                        GLib.idle_add(monitored.callback, monitored.args)
                    else:
                        GLib.idle_add(monitored.callback)

monitor = FileMonitor()
